﻿/*
 Copyright (c) 2013, Earth People Technology. All rights reserved.
 http://www.earthpeopletechnology.com

 Permission to use, copy, modify, and/or distribute this software for any purpose with or without fee is hereby granted, provided that the above copyright notice and this permission 
 notice appear in all copies.

 THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. 
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, 
 WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;
using System.Diagnostics;
using System.Text.RegularExpressions;
using Microsoft.Win32;


namespace EPT_Transfer_Demo
{
    public partial class EPT_Transfer_Demo : Form
    {

        public EPT_Transfer_Demo()
        {
            InitializeComponent();

            btnBlkCompare8.Enabled = false;
            btnBlkCompare16.Enabled = false;
            btnTrigger1.Enabled = false;
            btnTrigger2.Enabled = false;
            btnTrigger3.Enabled = false;
            btnTrigger4.Enabled = false;
            btnLEDReset.Enabled = false;
            btnUserBlk.Enabled = false;
            btnWriteByte.Enabled = false;
            tbWriteMultiByte.Enabled = false;
            btnWriteMultiByte.Enabled = false;
            btnBlockSendRst.Enabled = false;
            btnTransferReset.Enabled = false;
            btnResetTriggerOut.Enabled = false;
            btnResetBlock.Enabled = false;
            btnStopBlockTransfer.Enabled = false;
            btnLoopBack.Enabled = false;

            for (int i = 0; i < EPTTransmitDevice.Length; ++i)
            {
                EPTTransmitDevice[i] = new Transfer();
            }

            rbtnRepitions.Checked = true;
            rbtnInfinite.Checked = false;
        }

        //Sting array to hold the Port names that match the VID = "403 PID = "6010"
        public string[] MatchingComPortList = new string[10];

        // Local Serial Port parameters From Properties Page
        public static string PortName = "COM4";
        public static string BaudRate = "115200";
        public static string DataBits = "8";
        public static string vParity = "None";
        public static string StopBits = "1";
        public static string pHandshake = "None";

        // Serial port send variables
        byte[] PutBuff;
        char[] PutChar;

        //Flag  for converting incoming stream to Hex
        public static bool HexConversion = false;
        public static int IdxHexChar = 0;
        public static bool hexConversionTabFlag = false;

        //Line Feed
        public static bool EchoTransmitCharacters = true;
        public static bool AppendLineFeed = false;

        //EPT Receive Command Parameters
        public const byte TRIGGER_IN_COMMAND = 0x08;
        public const byte TRANSFER_IN_COMMAND = 0x10;
        public const byte BLOCK_IN_COMMAND = 0x20;

        // EPT EndTerm parameters
        public const byte TRANSFER_ENDTERM_1 = 0x01;
        public const byte TRANSFER_ENDTERM_2 = 0x02;
        public const byte TRANSFER_ENDTERM_3 = 0x03;
        public const byte TRANSFER_ENDTERM_4 = 0x04;
        public const byte TRANSFER_ENDTERM_5 = 0x05;
        public const byte TRANSFER_ENDTERM_6 = 0x06;
        public const byte TRANSFER_ENDTERM_7 = 0x07;

        //Parameters for EPT Receive object Trigger
        public const byte TRIGGER_IN_BYTE_1 = 0x01;
        public const byte TRIGGER_IN_BYTE_2 = 0x02;
        public const byte TRIGGER_IN_BYTE_4 = 0x04;
        public const byte TRIGGER_IN_BYTE_8 = 0x08;
        public const byte TRIGGER_IN_BYTE_10 = 0x10;
        public const byte TRIGGER_IN_BYTE_20 = 0x20;
        public const byte TRIGGER_IN_BYTE_40 = 0x40;
        public const byte TRIGGER_IN_BYTE_80 = 0x80;

        //Parameters for EPT Receive object Transfer byte
        public const byte TRANSFER_IN_ADDRESS_1 = 0x1;
        public const byte TRANSFER_IN_ADDRESS_2 = 0x2;

        //Parameters for EPT Receive object Block
        public const byte BLOCK_IN_ADDRESS_1 = 0x1;
        public const byte BLOCK_IN_ADDRESS_2 = 0x2;

        //Transfer Parameters
        public const byte CONTROL_REG_RESET = 0x00;
        public const byte TRANSFER_LOOPBACK = 0x01;
        public const byte BLOCK_LOOPBACK = 0x02;
        public const byte LOAD_IMAGE_TO_BUFFER = 0x04;
        public const byte LOAD_FACE_IMAGE = 0x08;
        public const byte RANDOM_IMAGE = 0x10;
        public const byte SET_SHIFT_COUNT_UP = 0x40;
        public const byte SET_SHIFT_COUNT_DOWN = 0x50;
        public const byte SET_STATIC_IMAGE = 0x70;
        public const byte SET_SHIFT_RIGHT_IMAGE = 0x90;
        public const byte SET_SHIFT_LEFT_IMAGE = 0xa0;

        //Index to store device selection
        //Int32 device_index;

        //Create a Receive object of the Transfer Class.
        //Transfer EPTReceiveData = new Transfer();

        //Eight Byte block for Block Transfers
        byte[] block_in_payload;

        //Create an array of the Transfer Class for device
        Transfer[] EPTTransmitDevice = new Transfer[8];

        //Block Transfer and Loop counter
        uint BlockCount = 0;
        uint LoopCount = 0;

        //Block Transfer Control 
        public bool BlockTransferInfinite = false;
        public bool BlockTransferReps = false;
        public bool BlockTransferStop = false;

        //String to Hold payload
        public String PayloadString = "";

        //Value to hold the errors in Block Compares
        public int BlockErrorCount = 0;
        public bool EnableLogFile = false;

        //Hold the previous status of the LED
        int LEDStatusPrevious = 0;

        //LED Status Register
        byte LEDStatus;

        //Parameters
        public const int SELECT_RANDOM = 1;
        public const int SELECT_SHIFT_LEFT = 2;
        public const int SELECT_SHIFT_RIGHT = 3;
        public const int SELECT_COUNT_UP = 4;
        public const int SELECT_COUNT_DOWN = 5;

        //Parameters for EPT Receive object
        public const byte TRANSFER_OUT_ADDRESS_1 = 0x1;
        public const byte TRANSFER_OUT_ADDRESS_2 = 0x2;

        // Main object loader
        private void EPT_Transfer_Demo_Load(object sender, System.EventArgs e)
        {

            //String buffer
            String PortText = "";

            //Index registers
            int Index = 0, EPTgroupNumber=0;

            // Call the List Devices function
            List<string> names = ComPortNames("0403", "6010");

            // Get a list of serial port names.
            string[] ports;
            ports = SerialPort.GetPortNames();


            if (names.Count > 0)
            {
                foreach (String port in ports)
                {
                    //Compare port name with the found VID/PID 
                    //combinations. Add them to Matching port list
                    //and comboDevList
                    if (names.Contains(port))
                    {
                        MatchingComPortList[Index] = port;

                        if (Index == 0)
                        {
                            PortText = "EPT JTAG Blaster " + EPTgroupNumber;
                            Index++;
                        }
                        else
                        {
                            PortText = "EPT Serial Communications " + EPTgroupNumber++;
                            Index++;
                        }
                        cmbDevList.Items.Add(PortText);
                    }
                }
            }
            else
                MessageBox.Show("No EPT Devices found!");


            SetButtonEnables_Close();
            //SetButtonEnables_Stop();

        }


        private void SetButtonEnables_Open()
        {
            //Byte Transfer Buttons
                btnWriteByte.Enabled = true;
                tbWriteMultiByte.Enabled = true;
                btnWriteMultiByte.Enabled = true;
                
            //LED Blinky Buttons
                btnTrigger1.Enabled = true;
                btnTrigger2.Enabled = true;
                btnTrigger3.Enabled = true;
                btnTrigger4.Enabled = true;
                btnLEDReset.Enabled = true;
                btnRandom.Enabled = true;
                btnShiftLeft.Enabled = true;
                btnShiftRight.Enabled = true;
                btnShiftUp.Enabled = true;
                btnShiftDown.Enabled = true;

            //Block Transfer Buttons
                btnBlkCompare8.Enabled = true;
                btnBlkCompare16.Enabled = true;
                btnUserBlk.Enabled = true;
                btnBlockSendRst.Enabled = true;
                btnTransferReset.Enabled = true;
                btnResetTriggerOut.Enabled = true;
                btnResetBlock.Enabled = true;
                btnStopBlockTransfer.Enabled = true;
                btnLoopBack.Enabled = true;

                // Marks the device as closed in the GUI
                btnOpenDevice.Enabled = false;
                btnCloseDevice.Enabled = true;
        }

        private void SetButtonEnables_Close()
        {

            btnLoopBack.Enabled = true;
            btnTrigger1.Enabled = true;
            btnTrigger2.Enabled = true;
            btnTrigger3.Enabled = true;
            btnTrigger4.Enabled = true;
            btnLEDReset.Enabled = true;
            btnBlkCompare8.Enabled = true;
            btnBlkCompare16.Enabled = true;
            btnStopBlockTransfer.Enabled = true;
            btnResetBlock.Enabled = true;
            btnResetTriggerOut.Enabled = true;
            gbTriggerOut.Enabled = true;
            gbTransferControl.Enabled = true;
            btnTransferReset.Enabled = true;
            btnBlockSendRst.Enabled = true;
            tbBlockSend.Enabled = true;

            // Make sure the device is closed
            if (!serialPort_AH.IsOpen)
            {
                btnBlkCompare8.Enabled = false;
                btnBlkCompare16.Enabled = false;
                btnTrigger1.Enabled = false;
                btnTrigger2.Enabled = false;
                btnTrigger3.Enabled = false;
                btnTrigger4.Enabled = false;
                btnLEDReset.Enabled = false;
                btnUserBlk.Enabled = false;
                btnWriteByte.Enabled = false;
                tbWriteMultiByte.Enabled = false;
                btnWriteMultiByte.Enabled = false;
                btnBlockSendRst.Enabled = false;
                btnTransferReset.Enabled = false;
                btnResetTriggerOut.Enabled = false;
                btnResetBlock.Enabled = false;
                btnStopBlockTransfer.Enabled = false;
                btnLoopBack.Enabled = false;
            }

            // Marks the device as closed in the GUI
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;
        }


        private void cmbDevList_SelectedIndexChanged_1(object sender, EventArgs e)
        {
            //Get Port name from Combo Box
            PortName = MatchingComPortList[cmbDevList.SelectedIndex];
        }
        private void btnOpenDevice_Click(object sender, EventArgs e)
        {
            OpenSerialPort1();
            if (serialPort_AH.IsOpen)
            {
                lblDeviceConnected.Text = "Device Connected";
                SetButtonEnables_Open();
            }

            //Send the load EPT Image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_IMAGE_TO_BUFFER);

            Thread.Sleep(5);

            //Send the load EPT Image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_IMAGE_TO_BUFFER);

            Thread.Sleep(5);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_LEFT_IMAGE);

            Thread.Sleep(5);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_LEFT_IMAGE);

            //Load the timer in the FPGA with the value in the textbox
            LoadTimerValue();
        }

        // Handle "Close" device button
        private void btnCloseDevice_Click(object sender, EventArgs e)
        {
            if (serialPort_AH.IsOpen)
            {
                CloseSerialPort1();
                lblDeviceConnected.Text = "Dis-Connected";
                SetButtonEnables_Close();
            }
        }

        // Handle the "Byte" button
        private void btnWriteByte_Click(object sender, EventArgs e)
        {
            int address_to_device;
            int ibyte = Convert.ToInt32(tbNumBytes.Text);
            address_to_device = Convert.ToInt32(tbAddress.Text);

            EPT_AH_SendByte((byte)address_to_device, (byte)ibyte);
        }

        // Handle the "Multi Byte" button
        private void btnWriteMultiByte_Click(object sender, EventArgs e)
        {
            int i, address_to_device;
            address_to_device = Convert.ToInt32(tbAddress.Text);
            string[] result = tbWriteMultiByte.Text.Split(' ');
            for (i = 0; i < result.Length; i++)
            {

                EPT_AH_SendByte((byte)address_to_device, (byte)Convert.ToInt32(result[i]));
                Thread.Sleep(1);
            }

        }

        // Handle the "LoopBack" button
        private void btnLoopBack_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, TRANSFER_LOOPBACK);

            Thread.Sleep(5);


            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, TRANSFER_LOOPBACK);


        }


        private void btnLEDReset_Click(object sender, EventArgs e)
        {
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_IMAGE_TO_BUFFER);
            Thread.Sleep(1);
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, CONTROL_REG_RESET);

            Thread.Sleep(500);
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_IMAGE_TO_BUFFER);
            Thread.Sleep(1);
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, CONTROL_REG_RESET);

            btnRandom.BackColor = Color.LightGray;
            btnShiftLeft.BackColor = Color.LightGray;
            btnShiftRight.BackColor = Color.LightGray;
            btnShiftUp.BackColor = Color.LightGray;
            btnShiftDown.BackColor = Color.LightGray;

            btnRandom.FlatStyle = FlatStyle.Standard;
            btnShiftLeft.FlatStyle = FlatStyle.Standard;
            btnShiftRight.FlatStyle = FlatStyle.Standard;
            btnShiftUp.FlatStyle = FlatStyle.Standard;
            btnShiftDown.FlatStyle = FlatStyle.Standard;
        }

        public void LoadTimerValue()
        {

            int TimerValue = 0;

            if (trkbrTimer.Value > 65535)
                TimerValue = 65535;
            else if (trkbrTimer.Value < 1000)
                TimerValue = 1000;
            else
                TimerValue = trkbrTimer.Value;

            tbTimerValue.Text = Convert.ToString(TimerValue);
            //Send the Timer Low Value to the Active Transfer Module 2
            int intValue = Convert.ToInt32(tbTimerValue.Text);
            byte[] intBytes = BitConverter.GetBytes(intValue);
            //if (BitConverter.IsLittleEndian)
            //    Array.Reverse(intBytes);
            byte[] result = intBytes;

            EPT_AH_SendByte(TRANSFER_ENDTERM_2, (byte)result[0]);

            //Send Trigger 4 to latch the Timer Low Value
            EPT_AH_SendTrigger(TRIGGER_IN_BYTE_10);

            //Send the Timer High Value to the Active Transfer Module 2
            EPT_AH_SendByte(TRANSFER_ENDTERM_2, (byte)result[1]);

            //Send Trigger 5 to latch the Timer High Value
            EPT_AH_SendTrigger(TRIGGER_IN_BYTE_20);
        }

        private void btnLoadEPTImage_Click(object sender, EventArgs e)
        {
            //Send the load EPT Image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_IMAGE_TO_BUFFER);

            Thread.Sleep(5);

            //Send the load EPT Image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_IMAGE_TO_BUFFER);

            Thread.Sleep(5);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);

            Thread.Sleep(5);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);

            //Load the timer in the FPGA with the value in the textbox
            LoadTimerValue();

        }

        private void btnStaticImage_Click(object sender, EventArgs e)
        {
            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);
        }

        private void btnLoadFace1_Click(object sender, EventArgs e)
        {
            //Send the Load EPT Image value to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_FACE_IMAGE);

            Thread.Sleep(5);

            //Send the static image command
            //Send the Load EPT Image value to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, LOAD_FACE_IMAGE);

            Thread.Sleep(5);


            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);

            Thread.Sleep(5);


            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);

            //Load the timer in the FPGA with the value in the textbox
            LoadTimerValue();
        }



        private void btnRandom_Click(object sender, EventArgs e)
        {
            byte SeedValue = 0;
            // Use a random timer instead of 100ms
            Random random = new Random();
            // Go between 50 and 300 milliseconds
            SeedValue = (byte)random.Next(10, 255);

            LoadTimerValue();

            //Send the Random Seed value to the Active Transfer Module 2	
            EPT_AH_SendByte(TRANSFER_ENDTERM_1, (byte)SeedValue);

            //Send Trigger 6 to latch the Seed Value
            EPT_AH_SendTrigger(TRIGGER_IN_BYTE_40);

            //Send the start Random Blinky to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, RANDOM_IMAGE);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_RANDOM);
        }

        private void btnShiftLeft_Click(object sender, EventArgs e)
        {
            //Send the start Shift Left to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_LEFT_IMAGE);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_SHIFT_LEFT);

            //Send the start Shift Left to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_LEFT_IMAGE);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_SHIFT_LEFT);
        }

        private void btnShiftRight_Click(object sender, EventArgs e)
        {
            //Send the start Shift Right to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_RIGHT_IMAGE);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_SHIFT_RIGHT);

            //Send the start Shift Right to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_RIGHT_IMAGE);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_SHIFT_RIGHT);
        }

        private void btnCountUp_Click(object sender, EventArgs e)
        {
            //Send the start Shift Right to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_COUNT_UP);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_COUNT_UP);

            //Send the start Shift Right to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_COUNT_UP);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_COUNT_UP);
        }

        private void btnCountDown_Click(object sender, EventArgs e)
        {
            //Send the start Shift Right to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_COUNT_DOWN);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_COUNT_DOWN);

            //Send the start Shift Right to the Control Register
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_SHIFT_COUNT_DOWN);

            //Set the Button State so User can know which function is selected
            SetLEDControlButtons(SELECT_COUNT_DOWN);
        }

        private void trkbrTimer_Scroll(object sender, EventArgs e)
        {
            LoadTimerValue();
        }

        void SetLEDControlButtons(int FuncSelect)
        {
            switch (FuncSelect)
            {
                case SELECT_RANDOM:
                    btnRandom.BackColor = Color.LightSteelBlue;
                    btnShiftLeft.BackColor = Color.LightGray;
                    btnShiftRight.BackColor = Color.LightGray;
                    btnShiftUp.BackColor = Color.LightGray;
                    btnShiftDown.BackColor = Color.LightGray;
                    break;
                case SELECT_SHIFT_LEFT:
                    btnRandom.BackColor = Color.LightGray;
                    btnShiftLeft.BackColor = Color.LightSteelBlue;
                    btnShiftRight.BackColor = Color.LightGray;
                    btnShiftUp.BackColor = Color.LightGray;
                    btnShiftDown.BackColor = Color.LightGray;
                    break;
                case SELECT_SHIFT_RIGHT:
                    btnRandom.BackColor = Color.LightGray;
                    btnShiftLeft.BackColor = Color.LightGray;
                    btnShiftRight.BackColor = Color.LightSteelBlue;
                    btnShiftUp.BackColor = Color.LightGray;
                    btnShiftDown.BackColor = Color.LightGray;
                    break;
                case SELECT_COUNT_UP:
                    btnRandom.BackColor = Color.LightGray;
                    btnShiftLeft.BackColor = Color.LightGray;
                    btnShiftRight.BackColor = Color.LightGray;
                    btnShiftUp.BackColor = Color.LightSteelBlue;
                    btnShiftDown.BackColor = Color.LightGray;
                    break;
                case SELECT_COUNT_DOWN:
                    btnRandom.BackColor = Color.LightGray;
                    btnShiftLeft.BackColor = Color.LightGray;
                    btnShiftRight.BackColor = Color.LightGray;
                    btnShiftUp.BackColor = Color.LightGray;
                    btnShiftDown.BackColor = Color.LightSteelBlue;
                    break;
                default:
                    btnRandom.BackColor = Color.LightGray;
                    btnShiftLeft.BackColor = Color.LightGray;
                    btnShiftRight.BackColor = Color.LightGray;
                    btnShiftUp.BackColor = Color.LightGray;
                    btnShiftDown.BackColor = Color.LightGray;
                    break;
            }

            btnRandom.FlatStyle = FlatStyle.Standard;
            btnShiftLeft.FlatStyle = FlatStyle.Standard;
            btnShiftRight.FlatStyle = FlatStyle.Standard;
            btnShiftUp.FlatStyle = FlatStyle.Standard;
            btnShiftDown.FlatStyle = FlatStyle.Standard;
        }
       
        //===============================
        //  Block Transfer Controls
        //===============================
        
        // Handle the radiobutton check for "Infinite" loop
        private void rbtnInfinite_CheckedChanged(object sender, EventArgs e)
        {
            BlockTransferInfinite = true;
            BlockTransferReps = false;
        }

        // Handle the radiobutton check for "Repetitions" loop
        private void rbtnRepitions_CheckedChanged(object sender, EventArgs e)
        {
            BlockTransferInfinite = false;
            BlockTransferReps = true;
        }

        // Handle the "Stop" block transfer button
        private void btnStopBlockTransfer_Click(object sender, EventArgs e)
        {
            BlockTransferStop = true;
        }

        // Reset the "Button XX Pressed" in Switches Groups
        private void btnResetTriggerOut_Click(object sender, EventArgs e)
        {

            lLabelSwitch1.Text = "";
            lLabelSwitch2.Text = "";

        }

        // Handle user block update
        private void btnUserBlk_Click(object sender, EventArgs e)
        {
            int value;
            if (tbBlockSend.Text.Trim().Length == 0) return;
            int BlockAddress = (int)Convert.ToInt32(tbBlockRcvAddress.Text);
            uint BlockRepitions = (uint)Convert.ToInt32(tbRepitions.Text);
            if (EPTTransmitDevice[BlockAddress].TransferPending)
                return;

            int i, address_to_device;
            address_to_device = Convert.ToInt32(tbAddress.Text);
            string[] result = tbBlockSend.Text.Split(' ', '\n');
            block_in_payload = new byte[result.Length];

            for (i = 0; i < result.Length; i++)
            {
                int.TryParse(result[i], out value);
                block_in_payload[i] = Convert.ToByte(value);// Convert.ToByte(result[i]);
            }

            EPTTransmitDevice[BlockAddress].Address = BlockAddress;
            EPTTransmitDevice[BlockAddress].Length = result.Length;
            EPTTransmitDevice[BlockAddress].Repititions = BlockRepitions;
            BlockCount = 0;
            LoopCount = 0;
            BlockErrorCount = 0;
            BlockTransferStop = false;

            Thread t = new Thread(new ParameterizedThreadStart(BlockCompare));
            t.Start(BlockAddress);

        }

        // Handle the "BLOCK8" compare button
        private void btnBlkCompare8_Click(object sender, EventArgs e)
        {
            int BlockAddress = (int)Convert.ToInt32(tbBlockRcvAddress.Text);
            int BlockLength = 8;
            uint BlockRepitions = (uint)Convert.ToInt32(tbRepitions.Text);
            if (EPTTransmitDevice[BlockAddress].TransferPending)
                return;
            EPTTransmitDevice[BlockAddress].Address = BlockAddress;
            EPTTransmitDevice[BlockAddress].Length = BlockLength;
            EPTTransmitDevice[BlockAddress].Repititions = BlockRepitions;
            BlockCount = 0;
            LoopCount = 0;
            BlockErrorCount = 0;
            BlockTransferStop = false;
            block_in_payload = block_8_in_payload;

            Thread t = new Thread(new ParameterizedThreadStart(BlockWrite));
            t.Start(BlockAddress);
        }

        // Handle the "BLOCK16" compare button
        private void btnBlkCompare16_Click(object sender, EventArgs e)
        {
            int BlockAddress = (int)Convert.ToInt32(tbBlockRcvAddress.Text);
            int BlockLength = 16;
            uint BlockRepitions = (uint)Convert.ToInt32(tbRepitions.Text);
            if (EPTTransmitDevice[BlockAddress].TransferPending)
                return;
            EPTTransmitDevice[BlockAddress].Address = BlockAddress;
            EPTTransmitDevice[BlockAddress].Length = BlockLength;
            EPTTransmitDevice[BlockAddress].Repititions = BlockRepitions;
            BlockCount = 0;
            LoopCount = 0;
            BlockErrorCount = 0;
            BlockTransferStop = false;
            block_in_payload = block_16_in_payload;

            Thread t = new Thread(new ParameterizedThreadStart(BlockWrite));
            t.Start(BlockAddress);

        }

        // Handle the "BLOCK256" compare button
        private void btnBlkCompare256_Click(object sender, EventArgs e)
        {
            int BlockAddress = (int)Convert.ToInt32(tbBlockRcvAddress.Text);
            int BlockLength = 128;
            uint BlockRepitions = (uint)Convert.ToInt32(tbRepitions.Text);
            if (EPTTransmitDevice[BlockAddress].TransferPending)
                return;
            EPTTransmitDevice[BlockAddress].Address = BlockAddress;
            EPTTransmitDevice[BlockAddress].Length = BlockLength;
            EPTTransmitDevice[BlockAddress].Repititions = BlockRepitions;
            BlockCount = 0;
            LoopCount = 0;
            BlockErrorCount = 0;
            BlockTransferStop = false;
            block_in_payload = block_128_in_payload;

            Thread t = new Thread(new ParameterizedThreadStart(BlockWrite));
            t.Start(BlockAddress);

        }

        // Compare blocks of send/receive data
        public unsafe void BlockWrite(object data)
        {
            int BlockAddress = (int)data;
            byte[] cBuf = new Byte[EPTTransmitDevice[BlockAddress].Length];

            //Copy the data to the buffer to be transferred.
            Buffer.BlockCopy(block_in_payload, 0, cBuf, 0, EPTTransmitDevice[BlockAddress].Length);
            //Send the data to the Active Device
            EPT_AH_SendBlock((byte)EPTTransmitDevice[BlockAddress].Address, (byte)EPTTransmitDevice[BlockAddress].Length, cBuf);
           //Send the Block Loop Back command
            //Thread.Sleep(1);
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, BLOCK_LOOPBACK);
            //Send the transfer FIFO to the Active Host command
            //Thread.Sleep(1);
            EPT_AH_SendTrigger((byte)128);

        }

        // Compare blocks of send/receive data
        public unsafe void BlockCompare(object data)
        {
            int BlockAddress = (int)System.Convert.ToUInt32(data);

            if ((EPTTransmitDevice[BlockAddress].Repititions > 0) & !EPTTransmitDevice[BlockAddress].TransferPending & !BlockTransferStop)
            {
                //Send the transfer FIFO to the Active Host command
                //Thread.Sleep(1);
                EPT_AH_SendTrigger((byte)128);

                if (BlockTransferInfinite)
                {
                    EPTTransmitDevice[BlockAddress].Repititions = 1;
                }
                else
                {
                    EPTTransmitDevice[BlockAddress].Repititions--;
                }
            }
        }


        // Handle the "OK" button
        private void btnOk_Click(object sender, EventArgs e)
        {
            CloseSerialPort1();
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            Application.Exit();
        }

        // Handle the "Cancel" button
        private void btnCancel_Click(object sender, EventArgs e)
        {
            CloseSerialPort1();
            btnOpenDevice.Enabled = true;
            btnCloseDevice.Enabled = false;

            Application.Exit();

        }

        // Handle clearing of the Block window
        private void btnResetBlock_Click(object sender, EventArgs e)
        {
            tbBlockRcv.Clear();
        }

        // Parse incoming EPT data
        private void EPTParseReceive()
        {
            // Parse the command
            switch (EPTReceiveDevice.Command)
            {
                // This is a trigger command
                case TRIGGER_IN_COMMAND:
                    TriggerOutReceive();
                    break;
                // This is a single-byte transfer command
                case TRANSFER_IN_COMMAND:
                    TransferOutReceive();
                    break;
                // This is a data block transfer command
                case BLOCK_IN_COMMAND:
                    BLockOutReceive();
                    break;
                default:
                    break;
            }
        }

        // Handle GUI update of an incoming trigger
        public void TriggerOutReceive()
        {
            //switch (EPTReceiveData.cBlockBuf[0])
            switch (EPTReceiveDevice.Payload)
            {
                case 0x04:
                    lLabelSwitch1.Text = "Switch 1\n Pressed";
                    break;
                case 0x08:
                    lLabelSwitch2.Text = "Switch 2\n Pressed";
                    break;
                default:
                    lLabelSwitch1.Text = "";
                    lLabelSwitch2.Text = "";
                    break;
            }
        }

        // Handle GUI update of an incoming byte transfer
        public void TransferOutReceive()
        {
                switch (EPTReceiveDevice.Address)
                {
                    case TRANSFER_OUT_ADDRESS_1:
                        TransferOutReceiveLoopBackByte();
                        break;

                    case TRANSFER_OUT_ADDRESS_2:
                        TransferOutReceiveLEDStatus();
                        break;
                    default:
                        break;
                }

        }

        public void TransferOutReceiveLEDStatus()
        {
            string WriteRcvChar = "";

            LEDStatus = EPTReceiveDevice.cBlockBuf[0];
            WriteRcvChar = String.Format("{0}", LEDStatus);

            if ((LEDStatus & 0x01) != (LEDStatusPrevious & 0x01))
            {
                if ((LEDStatus & 0x01) == 1) btnTrigger1.BackColor = Color.Lime;
                else btnTrigger1.BackColor = Color.LightGray;
            }

            if ((LEDStatus & 0x02) != (LEDStatusPrevious & 0x02))
            {
                if ((LEDStatus & 0x02) == 2) btnTrigger2.BackColor = Color.Lime;
                else btnTrigger2.BackColor = Color.LightGray;
            }

            if ((LEDStatus & 0x04) != (LEDStatusPrevious & 0x04))
            {
                if ((LEDStatus & 0x04) == 4) btnTrigger3.BackColor = Color.Lime;
                else btnTrigger3.BackColor = Color.LightGray;
            }

            if ((LEDStatus & 0x08) != (LEDStatusPrevious & 0x08))
            {
                if ((LEDStatus & 0x08) == 8) btnTrigger4.BackColor = Color.Lime;
                else btnTrigger4.BackColor = Color.LightGray;
            }

            LEDStatusPrevious = LEDStatus;

            btnTrigger1.FlatStyle = FlatStyle.Flat;
            btnTrigger2.FlatStyle = FlatStyle.Flat;
            btnTrigger3.FlatStyle = FlatStyle.Flat;
            btnTrigger4.FlatStyle = FlatStyle.Flat;
        }

        public void TransferOutReceiveLoopBackByte()
        {
            string WriteRcvChar = "";
            WriteRcvChar = String.Format("{0}", (int)EPTReceiveDevice.Payload);
            this.Invoke(new MethodInvoker(delegate() { tbDataBytes.AppendText(", " + WriteRcvChar); }));
        }

        // Handle clearing of byte data from the GUI when "Rst" is pressed
        private void btnTransferReset_Click(object sender, EventArgs e)
        {
            tbDataBytes.Clear();
        }

        // Handle GUI update of an incoming block transfer
        public void BLockOutReceive()
        {
            EPTTransmitDevice[EPTReceiveDevice.Address].TransferPending = false;
            BlockCount = BlockCount + EPTReceiveDevice.Length;
            BlockErrorCount += ByteArrayCompare(block_in_payload, EPTReceiveDevice.cBlockBuf, EnableLogFile);


            if (EPTTransmitDevice[EPTReceiveDevice.Address].Repititions == 0)
            {
                Thread u = new Thread(new ParameterizedThreadStart(DisplayBlockOut));
                u.Start(BlockCount);
            }
            else if (BlockTransferInfinite)
            {
                if ((LoopCount++ % 100) == 0)
                {
                    Thread u = new Thread(new ParameterizedThreadStart(DisplayBlockOut));
                    u.Start(BlockCount);
                }
            }

            Thread.Sleep(5);
            Thread t = new Thread(new ParameterizedThreadStart(BlockCompare));
            t.Start(EPTReceiveDevice.Address);
        
        }

        // Display data from the incoming block of transfer
        public void DisplayBlockOut(object CountData)
        {
            string s = String.Empty;
            foreach (byte b in EPTReceiveDevice.cBlockBuf)
            {
                s += String.Format("{0:x2}", (int)System.Convert.ToUInt32(b.ToString()));
                s += "\r\n";
            }
            //tbBlockRcv.AppendText(s);
            this.Invoke(new MethodInvoker(delegate() { tbBlockRcv.AppendText(s); }));

            string s2 = String.Empty;
            s2 = String.Format("{0:x2}", (int)System.Convert.ToUInt32(EPTReceiveDevice.Address.ToString()));
            this.Invoke(new MethodInvoker(delegate() { tbBlockRcvAddress.Text = s2; }));

            string s3 = String.Empty;
            s3 = String.Format("{0}", (int)EPTReceiveDevice.Length);
            this.Invoke(new MethodInvoker(delegate() { tbBlockRcvLength.Text = s3; }));

            string sBlockCount = String.Format("{0}", (int)System.Convert.ToUInt32(CountData.ToString()));
            this.Invoke(new MethodInvoker(delegate() { tbBlockCount.Text = sBlockCount; }));


            string sBlockErrorCount = String.Format("{0}", BlockErrorCount);
            this.Invoke(new MethodInvoker(delegate() { tbBlockThruPut.Text = sBlockErrorCount; }));

        }

        public void UpdateTextBlock()
        {
            string s = String.Empty;
            foreach (byte b in EPTReceiveDevice.cBlockBuf)
            {
                s += String.Format("{0:x2}", (int)System.Convert.ToUInt32(b.ToString()));
                s += "\r\n";
            }
            tbBlockRcv.AppendText(s);
        }

        // Compare byte data between two arrays and return the total number of differences
        public int ByteArrayCompare(byte[] a1, byte[] a2, bool EnableLogFile)
        {
            int ErrorCount = 0;
            int Length = a1.Length;

            // Get the shortest of the two arrays
            if (Length > a2.Length) Length = a2.Length;
            for (int i = 0; i < Length; i++)
            {
                if (a1[i] != a2[i])
                {
                    ErrorCount++;
                }
            }
            // Return total number of errors
            return ErrorCount;
        }

        // Reset the Block Send window 
        private void btnBlockSendRst_Click(object sender, EventArgs e)
        {
            tbBlockSend.Clear();
        }


        public unsafe void BlockImageWrite()
        {
            int BlockAddress = (int)2;
            EPTTransmitDevice[BlockAddress].Address = BlockAddress;
            EPTTransmitDevice[BlockAddress].Length = (int)32;
            byte[] cBuf = new Byte[EPTTransmitDevice[BlockAddress].Length];

            //Clear the previous command in the LED Array FSM
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, CONTROL_REG_RESET);


            //Copy the dummy data to the buffer to be transferred.
            Buffer.BlockCopy(block_image_payload, 0, cBuf, 0, EPTTransmitDevice[BlockAddress].Length);
            //fixed (byte* pBuf = cBuf)
            //{
                //Send the dummy data to the Active Device
             EPT_AH_SendBlock((byte)EPTTransmitDevice[BlockAddress].Address,(byte)EPTTransmitDevice[BlockAddress].Length, cBuf);
            //}

            Thread.Sleep(1);

            EPT_AH_SendBlock((byte)EPTTransmitDevice[BlockAddress].Address, (byte)EPTTransmitDevice[BlockAddress].Length, cBuf);
            //}

            Thread.Sleep(1);

            //Send the Block load image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, BLOCK_LOOPBACK);

            Thread.Sleep(1);

            //Send the Block load image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, BLOCK_LOOPBACK);

            Thread.Sleep(1);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);

            Thread.Sleep(1);

            //Send the static image command
            EPT_AH_SendTransferControlByte(TRANSFER_ENDTERM_2, SET_STATIC_IMAGE);


        }

        private void btnTrigger1_Click(object sender, EventArgs e)
        {
            //Flip bit 1
            block_image_payload[0] ^= (1 << 0);


            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();

            LoadTimerValue();

        }

        private void btnTrigger2_Click(object sender, EventArgs e)
        {
            //Flip bit 2
            block_image_payload[0] ^= (1 << 1);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();

            LoadTimerValue();

        }

        private void btnTrigger3_Click(object sender, EventArgs e)
        {
            //Flip bit 3
            block_image_payload[0] ^= (1 << 2);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();

            LoadTimerValue();

        }

        private void btnTrigger4_Click(object sender, EventArgs e)
        {
            //Flip bit 4
            block_image_payload[0] ^= (1 << 3);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();

            LoadTimerValue();

        }



        private void btnTrigger5_Click(object sender, EventArgs e)
        {
            //Flip bit 5
            block_image_payload[0] ^= (1 << 4);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger6_Click(object sender, EventArgs e)
        {
            //Flip bit 6
            block_image_payload[0] ^= (1 << 5);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger7_Click(object sender, EventArgs e)
        {
            //Flip bit 1 row 2
            block_image_payload[3] ^= (1 << 0);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger8_Click(object sender, EventArgs e)
        {
            //Flip bit 2 row 2
            block_image_payload[3] ^= (1 << 1);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger9_Click(object sender, EventArgs e)
        {
            //Flip bit 3 row 2
            block_image_payload[3] ^= (1 << 2);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger10_Click(object sender, EventArgs e)
        {
            //Flip bit 4 row 2
            block_image_payload[3] ^= (1 << 3);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger11_Click(object sender, EventArgs e)
        {
            //Flip bit 5 row 2
            block_image_payload[3] ^= (1 << 4);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger12_Click(object sender, EventArgs e)
        {
            //Flip bit 6 row 2
            block_image_payload[3] ^= (1 << 5);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger13_Click(object sender, EventArgs e)
        {
            //Flip bit 1 row 3
            block_image_payload[6] ^= (1 << 0);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger14_Click(object sender, EventArgs e)
        {
            //Flip bit 2 row 3
            block_image_payload[6] ^= (1 << 1);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger15_Click(object sender, EventArgs e)
        {
            //Flip bit 3 row 3
            block_image_payload[6] ^= (1 << 2);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger16_Click(object sender, EventArgs e)
        {
            //Flip bit 4 row 3
            block_image_payload[6] ^= (1 << 3);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void button7_Click(object sender, EventArgs e)
        {
            //Flip bit 5 row 3
            block_image_payload[6] ^= (1 << 4);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger18_Click(object sender, EventArgs e)
        {
            //Flip bit 6 row 3
            block_image_payload[6] ^= (1 << 5);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger19_Click(object sender, EventArgs e)
        {
            //Flip bit 1 row 4
            block_image_payload[9] ^= (1 << 0);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger20_Click(object sender, EventArgs e)
        {
            //Flip bit 2 row 4
            block_image_payload[9] ^= (1 << 1);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger21_Click(object sender, EventArgs e)
        {
            //Flip bit 3 row 4
            block_image_payload[9] ^= (1 << 2);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger22_Click(object sender, EventArgs e)
        {
            //Flip bit 4 row 4
            block_image_payload[9] ^= (1 << 3);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void bnTrigger23_Click(object sender, EventArgs e)
        {
            //Flip bit 5 row 4
            block_image_payload[9] ^= (1 << 4);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger24_Click(object sender, EventArgs e)
        {
            //Flip bit 6 row 4
            block_image_payload[9] ^= (1 << 5);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger25_Click(object sender, EventArgs e)
        {
            //Flip bit 1 row 5
            block_image_payload[12] ^= (1 << 0);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger26_Click(object sender, EventArgs e)
        {
            //Flip bit 2 row 5
            block_image_payload[12] ^= (1 << 1);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger27_Click(object sender, EventArgs e)
        {
            //Flip bit 3 row 5
            block_image_payload[12] ^= (1 << 2);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger28_Click(object sender, EventArgs e)
        {
            //Flip bit 4 row 5
            block_image_payload[12] ^= (1 << 3);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger29_Click(object sender, EventArgs e)
        {
            //Flip bit 5 row 5
            block_image_payload[12] ^= (1 << 4);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger30_Click(object sender, EventArgs e)
        {
            //Flip bit 6 row 5
            block_image_payload[12] ^= (1 << 5);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger31_Click(object sender, EventArgs e)
        {
            //Flip bit 1 row 6
            block_image_payload[15] ^= (1 << 0);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger32_Click(object sender, EventArgs e)
        {
            //Flip bit 2 row 6
            block_image_payload[15] ^= (1 << 1);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger33_Click(object sender, EventArgs e)
        {
            //Flip bit 3 row 6
            block_image_payload[15] ^= (1 << 2);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger34_Click(object sender, EventArgs e)
        {
            //Flip bit 4 row 6
            block_image_payload[15] ^= (1 << 3);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger35_Click(object sender, EventArgs e)
        {
            //Flip bit 5 row 6
            block_image_payload[15] ^= (1 << 4);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

        private void btnTrigger36_Click(object sender, EventArgs e)
        {
            //Flip bit 6 row 6
            block_image_payload[15] ^= (1 << 5);

            //Send the updated image block to the DPL using Block Writes
            BlockImageWrite();


            LoadTimerValue();

        }

    }

    // Transfer object definition
    public class Transfer
    {
        public int Command;
        public int Address;
        public int Length;
        public int Payload;
        public byte[] cBlockBuf;
        public bool TransferPending;
        public uint Repititions;

        public Transfer()
        {
            Command = 0;
            Address = 0;
            Length = 0;
            Payload = 0;
            TransferPending = false;
            Repititions = 0;
        }

    }

}

